﻿using Photon.Pun;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace Kryz.CharacterStats.Examples
{
    public class PlayerCharacterStats : CharacterStats
    {
        [Header("Listeners:")]
        [SerializeField] private GameEventListener_Float experienceOnNPCDeath;
        [Header("Components:")]
        [SerializeField] Inventory inventory;
        [SerializeField] EquipmentPanel equipmentPanel;
        [SerializeField] StatPanel statPanel;

        [HideInInspector]
        public RiftPlayer player;

        public Level level;

        PhotonView photonView;

        Health health;
        Mana mana;

        readonly object allocatedSource = "Allocated";

        [Header("Runtime Data:")]
        public List<int> AllocatedStatPoints = new List<int>();

        public int AvailablePointsToAllocate;

        PlayerData playerLevelData = new PlayerData();
        string playerLevelKey;

        protected override void Awake()
        {
            player = GetComponentInParent<RiftPlayer>();
            photonView = GetComponentInParent<PhotonView>();
            health = player.GetComponent<Health>();
            mana = player.GetComponent<Mana>();

            playerLevelKey = GameStatePersistenceManager.Instance.playerLevelKey;

            if (GameStatePersistenceManager.Instance.HasDataForKey(playerLevelKey))
            {
                playerLevelData = GameStatePersistenceManager.Instance.LoadData<PlayerData>(playerLevelKey);
                level = new Level(playerLevelData.currentLevel, playerLevelData.currentExperience);
            }
            else
                level = new Level();

            base.Awake();

            if (!photonView.IsMine) return;

            for (int i = 0; i < statsDictionary.Keys.Count; i++)
            {
                AllocatedStatPoints.Add(0);
            }
            AvailablePointsToAllocate = 1;

            if (statPanel == null)
                statPanel = FindObjectOfType<StatPanel>();

            statPanel.SetPlayerStats(this);
            statPanel.SetStats(Strength, Agility, Intelligence, Spirit, Vitality);
            statPanel.UpdateStatValues();
            statPanel.SetCharacterInfo(photonView.Owner.NickName, level.currentLevel.ToString());
            statPanel.ToggleAllocateButtonsInteractable(AvailablePointsToAllocate > 0);
            onUpdateStatValues.AddListener(statPanel.UpdateStatValues);
            onUpdateStatValues.Invoke();

            if (inventory == null)
                inventory = FindObjectOfType<Inventory>();

            inventory.OnItemRightClickedEvent.AddListener(EquipFromInventory);

            if (equipmentPanel == null)
                equipmentPanel = FindObjectOfType<EquipmentPanel>();

            equipmentPanel.OnItemRightClickedEvent.AddListener(UnequipFromEquipPanel);

            level.OnLevelUpEvent.AddListener(OnLevelUp);
            level.OnExperienceChanged.AddListener(OnExperienceChanged);
            experienceOnNPCDeath.Response.AddListener(level.GainExperience);
        }

        private void OnExperienceChanged()
        {
            playerLevelData.currentLevel = level.currentLevel;
            playerLevelData.currentExperience = level.GetCurrentExperience();

            GameStatePersistenceManager.Instance.SaveData(GameStatePersistenceManager.Instance.playerLevelKey, playerLevelData);
        }

        private void OnLevelUp()
        {
            UpdateLevelOnOthers();

            statPanel.UpdateLevelInfo(level.currentLevel.ToString());

            IncreaseAllStatPoints(1);

            AvailablePointsToAllocate += 2;

            statPanel.ToggleAllocateButtonsInteractable(AvailablePointsToAllocate > 0);

            health.ChangeValue(health.GetMaxValue() * 0.2f);
            mana.ChangeValue(mana.GetMaxValue() * 0.2f);
        }

        private void EquipFromInventory(Item item)
        {
            if (item is EquippableItem)
            {
                Equip((EquippableItem)item);
            }
        }

        private void UnequipFromEquipPanel(Item item)
        {
            if (item is EquippableItem)
            {
                Unequip((EquippableItem)item);
            }
        }

        public void Equip(EquippableItem item)
        {
            if (inventory.RemoveItem(item))
            {
                ItemTooltip.Instance.HideTooltip();

                EquippableItem previousItem;
                if (equipmentPanel.AddItem(item, out previousItem))
                {
                    if (previousItem != null)
                    {
                        inventory.AddItem(previousItem);
                        previousItem.Unequip(this);
                        statPanel.UpdateStatValues();
                    }
                    item.Equip(this);
                    statPanel.UpdateStatValues();
                    onUpdateStatValues.Invoke();
                }
                else
                {
                    inventory.AddItem(item);
                }
            }
        }

        public void Unequip(EquippableItem item)
        {
            if (!inventory.IsFull() && equipmentPanel.RemoveItem(item))
            {
                item.Unequip(this);
                statPanel.UpdateStatValues();
                inventory.AddItem(item);
                onUpdateStatValues.Invoke();
            }
        }

        public void UpdateLevelOnOthers()
        {
            if (photonView.IsMine)
                photonView.RPC(nameof(RPC_OnLevelUp), RpcTarget.OthersBuffered, this.level.currentLevel);
        }

        [PunRPC]
        public void RPC_OnLevelUp(int level)
        {
            this.level.currentLevel = level;
            this.level.OnLevelUpEvent.Invoke();
        }

        public void UpdateAllocatedStats()
        {
            Strength.RemoveAllModifiersFromSource(allocatedSource);
            Agility.RemoveAllModifiersFromSource(allocatedSource);
            Intelligence.RemoveAllModifiersFromSource(allocatedSource);
            Spirit.RemoveAllModifiersFromSource(allocatedSource);
            Vitality.RemoveAllModifiersFromSource(allocatedSource);

            Strength.AddModifier(new StatModifier(AllocatedStatPoints[0], StatModType.Flat, allocatedSource));
            Agility.AddModifier(new StatModifier(AllocatedStatPoints[1], StatModType.Flat, allocatedSource));
            Intelligence.AddModifier(new StatModifier(AllocatedStatPoints[2], StatModType.Flat, allocatedSource));
            Spirit.AddModifier(new StatModifier(AllocatedStatPoints[3], StatModType.Flat, allocatedSource));
            Vitality.AddModifier(new StatModifier(AllocatedStatPoints[4], StatModType.Flat, allocatedSource));

            onUpdateStatValues.Invoke();
        }
    }
}
